---
title: Manage users
---

The following topics are for the basic management of users: how to create, modify, delete or deactivate users, and using a recovery email.

### Create a user

> If you want to automate user creation, you can do that either by [invitations](./invitations.md), [`user_write` stage](../../flow/stages/user_write), or [using the API](/developer-docs/api/browser).

1. In the Admin interface of your authentik instance, select **Directory > Users** in the left side menu.

2. Select the folder where you want to create a user.

3. Click **Create** (for a default user).

4. Fill in the required fields:

-   **Username**: This value must be unique across your user folders.
-   **Path**: The path where the user will be created. It will be automatically populated with the folder you selected in the previous step.

5. Fill the **_optional_** fields if needed:

-   **Name**: The display name of the user.
-   **Email**: The email address of the user. That will be used if there is a [notification rule](../../events/notifications) triggered or for [email stages](../../flow/stages/email).
-   **Is active**: Define is the newly created user account is active. Selected by default.
-   **Attributes**: Custom attributes definition for the user, in YAML or JSON format. These attributes can be used to enforce additional prompts on authentication stages or define conditions to enforce specific policies if the current implementation does not fit your use case. The value is an empty dictionary by default.

6. Click **Create**

You should see a confirmation pop-up on the top-right of the screen that the user has been created, and see the new user in the user list. You can directly click the username if you want to [modify your user](./user_basic_operations#modify-a-user).

### View user details

In the **Directory > Users** menu of the Admin interface, you can browse all the users in your authentik instance.

To view details about a specific user:

1. In the list of all users, click on the name of the user you want to check.

    This takes you to the **Overview** tab, with basic information about the user, and also quick access to perform basic actions to the user.

2. To see further details, click any of the other tabs:

-   **Session** shows the active sessions established by the user. If there is any need, you can clean up the connected devices for a user by selecting the device(s) and then clicking **Delete**. This forces the user to authenticate again on the deleted devices.
-   **Groups** allows you to manage the group membership of the user. You can find more details on [groups](../group).
-   **User events** displays all the events generated by the user during a session, such as login, logout, application authorisation, password reset, user info update, etc.
-   **Explicit consent** lists all the permissions the user has given explicitly to an application. Entries will only appear if the user is validating an [explicit consent flow in an OAuth2 provider](../../providers/oauth2/). If you want to delete the explicit consent (because the application is requiring new permissions, or the user has explicitly asked to reset his consent on third-party apps), select the applications and click **Delete**. The user will be asked to again give explicit consent to share information with the application.
-   **OAuth Refresh Tokens** lists all the OAuth tokens currently distributed. You can remove the tokens by selecting the applications and then clicking **Delete**.
-   **MFA Authenticators** shows all the authentications that the user has registered to their user profile. You can remove the tokens if the user has lost their authenticator and want to enroll a new one.

## Modify a user

After the creation of the user, you can edit any parameter defined during the creation.

To modify a user object, go to **Directory > Users**, and click the edit icon beside the name.

You can also go into [user details](#view-user-details), and click **Edit**.

## User recovery

If a user has lost their credentials, there are several options.

### Email them a recovery link

1. In the Admin interface, navigate to **Directory > Users** to display all users.

2. Either click the name of the user to display the full User details page, or click the chevron (the › symbol) beside their name to expand the toptions.

3. To generate a recovery link, which you can then copy and paste into an email, click **View recovery link**.

    A pop-up will appear on your browser with the link for you to copy and to send to the user.

### Automate email to a user

You can use our automated email to send a link with the URL for the user to reset their password. This option will only work if you have properly [configured a SMTP server during the installation](../../installation/docker-compose#email-configuration-optional-but-recommended) and set an email address for the user.

1. In the Admin interface, navigate to **Directory > Users** to display all users.

2. Either click the name of the user to display the full User details page, or click the chevron beside their name to expand the toptions.

3. To send the automated email to the user, click **Email recovery link**.

If the user does not receive the email, check if the mail server parameters [are properly configured](../../troubleshooting/emails).

### Reset the password for the user

As an Admin, you can simply reset the password for the user.

1. In the Admin interface, navigate to **Directory > Users** to display all users.

2. Either click the name of the user to display the full User details page, or click the chevron beside their name to expand the toptions.

3. To reset the user's password, click **Reset password**, and then define the new value.

## Deactivate or Delete user

#### To deactivate a user:

1. Go into the user list or detail, and click **Deactivate**.

2. Review the changes and click **Update**.

The active sessions are revoked and the authentication of the user blocked. You can reactivate the account by following the same procedure.

#### To delete a user:

:::caution
This deletion is not reversible, so be sure you do not need to recover any identity data of the user.
You may instead deactivate the account to preserve identity data.
:::

1. Go into the user list and select one (or multiple users) to delete and click **Delete** on the top-right of the page.

2. Review the changes and click **Delete**.

The user list refreshes and no longer displays the removed users.
